/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.fleet.machine.MachineDataService;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineProperty;
import org.netbeans.api.project.Project;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.Exceptions;

/**
 * This action can be called to start the machine creation wizard.
 * Implementations for different machine types should extends this class.
 *
 * @author Wytse Visser
 */
public abstract class MachineWizardAction extends AbstractAction implements ActionListener {

    private Project project;

    /**
     * Creates a new MachineWizardAction.
     *
     * @param project The project the action belongs to.
     * @param name The name of the action. (e.g. "New Machine")
     */
    public MachineWizardAction(Project project, String name) {
        this.project = project;
        putValue(NAME, name);
    }

    /**
     * Performs this action. Starts a new machine creation wizard and creates
     * the machine when the wizard finishes.
     *
     * @param e The event calling this action.
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        WizardDescriptor wiz = new WizardDescriptor(getWizardFactory().createWizardIterator());
        // {0} will be replaced by WizardDescriptor.Panel.getComponent().getName()
        // {1} will be replaced by WizardDescriptor.Iterator.name()
        wiz.setTitleFormat(new MessageFormat("{0} ({1})"));
        wiz.setTitle(getValue(NAME).toString());
        if (DialogDisplayer.getDefault().notify(wiz) == WizardDescriptor.FINISH_OPTION) {
            try {
                createMachine(wiz);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
    }

    protected Project getProject() {
        return project;
    }

    /**
     * Returns the value of the given property in the wizard descriptor. Falls
     * back to the value in the model if the property is not set in the wizard
     * descriptor.
     *
     * @param propertyName The name of the property to read the value for
     * @param wiz The wizard descriptor the property is initally read from
     * @param model The model to fall back to
     * @return The stored value for the property
     */
    protected static Object readProperty(String propertyName, WizardDescriptor wiz, MachineDataService.Model model) {
        Object value = wiz.getProperty(propertyName);
        if (value == null) {
            value = model.getProperty(propertyName);
        }
        return value;
    }

    /**
     * Creates and returns the MachineWizardImplementation for this action.
     *
     * @return The MachineWizardImplementation for this action
     */
    protected abstract MachineWizardFactory getWizardFactory();

    /**
     * Create and save a new machine with the values stored in the given
     * WizardDescriptor.
     *
     * @param wiz The WizardDescriptor with the values the user entered using
     * the wizard.
     * @throws IOException If somehow the machine can't be saved.
     */
    protected abstract void createMachine(WizardDescriptor wiz) throws IOException;

    /**
     * Adds dynamic properties stored in the WizardDescriptor to the machine.
     *
     * @param machine The machine
     * @param wiz The wizard descriptor with the dynamic properties
     */
    protected void addDynamicProperties(Machine machine, WizardDescriptor wiz) {
        Collection<List<MachineProperty>> propertyLists = MachineDynamicWizardPanel.getDynamicPropertyMap(wiz).values();
        Iterator it = propertyLists.iterator();
        while (it.hasNext()) {
            List<MachineProperty> propertyList = (List<MachineProperty>) it.next();
            for (MachineProperty property : propertyList) {
                machine.addProperty(property);
            }
        }
    }
}
