/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.util.List;
import nl.cloudfarming.client.fleet.machine.MachineDataService;
import nl.cloudfarming.client.fleet.model.MachineProvider;
import org.netbeans.api.project.Project;
import org.openide.WizardDescriptor;

/**
 * This factory provides functionality for the creation of objects used by the
 * machine creation wizard. Create an implementation for this factory to
 * customize the machine wizard for different machine types.
 *
 * @author Wytse Visser
 */
public abstract class MachineWizardFactory {

    private Project project;

    /**
     * Creates a MachineWizardFactory for the given project
     *
     * @param project The project
     */
    public MachineWizardFactory(Project project) {
        this.project = project;
    }

    /**
     * @return The project which belongs to this factory
     */
    protected Project getProject() {
        return project;
    }

    /**
     * Creates a new WizardIterator for the wizard.
     *
     * @return A WizardIterator for the machine wizard.
     */
    public MachineWizardIterator createWizardIterator() {
        return new MachineWizardIterator(this);
    }

    /**
     * Creates an overview panel for a machine wizard. This panel is added to
     * the last step in the machine creation wizard.
     *
     * @return The created overview panel
     */
    public abstract MachineWizardPanel createOverviewPanel();

    /**
     * Creates the panels which are initially added to the machine creation
     * wizard.
     *
     * @return The list with the initial panels.
     */
    public abstract List<WizardDescriptor.Panel<WizardDescriptor>> createInitialPanels();

    /**
     * @return The class that is implemented by data services for this machine
     * type.
     */
    public abstract Class<? extends MachineDataService> getDataServiceClass();

    /**
     * @return The class of the specific implementation of MachineProvider for
     * this machine
     */
    public abstract Class<? extends MachineProvider> getMachineProviderClass();
}
